// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund.Components.ImpactCooling;

interface

uses Esmf,
     Fund.CommonDimensions;

type
  
  IImpactCoolingState = public interface
    property cooling: IVariable2Dimensional<Timestep, Region, double> read;
    
    property cebm: IParameter1Dimensional<Region, double> read;
    property gdp90: IParameter1Dimensional<Region, double> read;
    
    property population: IParameter2Dimensional<Timestep, Region, double> read;
    property pop90: IParameter1Dimensional<Region, double> read;
    
    property income: IParameter2Dimensional<Timestep, Region, double> read;
    property ceel: Double read;
    
    property temp: IParameter1Dimensional<Timestep, double> read;
    property cenl: Double read;
        
    property cumaeei: IParameter2Dimensional<Timestep, Region, double> read;    
  end;
  
  ImpactCoolingComponent = public class
  private
  protected
  public
    method Run(clock: Clock; state: IImpactCoolingState);        
  end;
  
implementation

method ImpactCoolingComponent.Run(clock: Clock; state: IImpactCoolingState); 
begin
  var s:=state;
  var t:=clock.Current;

  for each r: Region in Enum.GetValues(typeof(Region)) do
  begin  
    var ypc: Double:=s.income[t,r]/s.population[t,r]*1000.0;
    var ypc90: Double:=s.gdp90[r]/s.pop90[r]*1000.0;
    
    s.cooling[t,r]:=s.cebm[r]*s.cumaeei[t,r]*s.gdp90[r]*Math.Pow(s.temp[t]/1.0,s.cenl)*Math.Pow(ypc/ypc90,s.ceel)*s.population[t,r]/s.pop90[r];
  end;  

end;

end.