// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund.Components.ImpactSeaLevelRise;

interface

uses Esmf,
     Fund.CommonDimensions;

type
  
  IImpactSeaLevelRiseState = public interface
    property wetval: IVariable2Dimensional<Timestep, Region, double> read;
    property wvbm: IParameter1Dimensional<Region, double> read;
    property income: IParameter2Dimensional<Timestep, Region, double> read;
    property population: IParameter2Dimensional<Timestep, Region, double> read;
    property gdp90: IParameter1Dimensional<Region,double> read;
    property pop90: IParameter1Dimensional<Region,double> read;
    property wetvalypcel: Double read;
    property wetlandscarcityparameter: Double read;
    property wetmax: IParameter1Dimensional<Region,double> read;
    property wettot: IParameter2Dimensional<Timestep,Region,double> read;
    
    property dryval: IVariable2Dimensional<Timestep, Region, double> read;
    property dvbm: IParameter1Dimensional<Region, double> read;
    property popdens: IParameter2Dimensional<Timestep,Region,double> read;
    // Income density in the OECD in 1990
    property incdens: double read;
    
    property npprotcost: IVariable2Dimensional<Timestep,Region,double> read;
    property pc: IParameter1Dimensional<Region,double> read;
    property ds: IParameter1Dimensional<Timestep,double> read;
    property discrate: Double read;
    
  end;
  
  ImpactSeaLevelRiseComponent = public class
  private
  protected
  public
    method Run(clock: Clock; state: IImpactSeaLevelRiseState);            
  end;
  
implementation

method ImpactSeaLevelRiseComponent.Run(clock: Clock; state: IImpactSeaLevelRiseState); 
begin
  var s:=state;
  var t:=clock.Current;

  for each r: Region in Enum.GetValues(typeof(Region)) do
  begin  
    var ypc:=1000.0*s.income[t,r]/s.population[t,r];
    var ypc90:=1000.0*s.gdp90[r]/s.pop90[r];
    var ygrowth:=s.income[t,r]/s.income[t-1,r] - 1.0;
    

  end;
end;

end.