// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund.Components.ScenarioUncertainty;

interface

uses
  Esmf,
  Fund.CommonDimensions;

type

  IScenarioUncertaintyState = public interface
    property pgrowth: IVariable2Dimensional<Timestep, Region, double> read;
    property ypcgrowth: IVariable2Dimensional<Timestep, Region, double> read;    

    property timeofuncertaintystart: Timestep read;
    
    property inputpgrowth: IParameter2Dimensional<Timestep, Region, double> read;
    property inputypcgrowth: IParameter2Dimensional<Timestep, Region, double> read;            
    property ecgradd: IParameter1Dimensional<Region, double> read;
    property pgadd: IParameter1Dimensional<Region, double> read;
  end;
  
  ScenarioUncertaintyComponent = public class
  private
  protected
  public
    method Run(clock: Clock; state: IScenarioUncertaintyState);        
  end;
  
implementation

method ScenarioUncertaintyComponent.Run(clock: Clock; state: IScenarioUncertaintyState); 
begin
  var s:=state;
  var t:=clock.Current; 

  for each r: Region in Enum.GetValues(typeof(Region)) do
  begin       
    s.ypcgrowth[t,r]:=s.inputypcgrowth[t,r] + iif(t>=s.timeofuncertaintystart, s.ecgradd[r]*double(t.Value-s.timeofuncertaintystart.value), 0.0);
    s.pgrowth[t,r]:=s.inputpgrowth[t,r] + iif(t>=s.timeofuncertaintystart, s.pgadd[r]*double(t.Value-s.timeofuncertaintystart.value), 0.0);
  end;  
end;

end.