// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

uses Esmf;

type
  
  IFundOld2Climate = public interface
    property addco2: IVariable1Dimensional<Timestep, double> read;
    property addch4: IVariable1Dimensional<Timestep, double> read;
    property addn2o: IVariable1Dimensional<Timestep, double> read;
    property addsf6: IVariable1Dimensional<Timestep, double> read;
  end; 
  
  FundOld2ClimateComponent = public class
  private
  protected
  public
    method Run(clock: Clock; state: IFundOld2Climate; parameters: FundParameter);        
  end;
  
implementation

method FundOld2ClimateComponent.Run(clock: Clock; state: IFundOld2Climate; parameters: FundParameter); 
begin  
  var t:=clock.Current;
  
  state.addch4[t]:=0;
  state.addco2[t]:=0;
  state.addn2o[t]:=0;
  state.addsf6[t]:=0;

  if (t.Value>=(parameters.emissionperiod-1950)) and (t.Value<(parameters.emissionperiod-1950+10)) then
  begin
    case parameters.marginalgas of
      0: state.addco2[t]:=1;
      1: state.addch4[t]:=1;
      2: state.addn2o[t]:=1;
      3: state.addsf6[t]:=1;
    end;
  end;  
 
end;

end.