// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

uses Esmf,
     Fund.CommonDimensions;

type
  
  IFundOld2ImpactAggregation = public interface
    property forests: IVariable2Dimensional<Timestep, Region, double> read;
    property drycost: IVariable2Dimensional<Timestep, Region, double> read;
    property protcost: IVariable2Dimensional<Timestep, Region, double> read;
    property entercost: IVariable2Dimensional<Timestep, Region, double> read;
    property hurrdam: IVariable2Dimensional<Timestep, Region, double> read;
    property deadcost: IVariable2Dimensional<Timestep, Region, double> read;
    property morbcost: IVariable2Dimensional<Timestep, Region, double> read;
    property wetcost: IVariable2Dimensional<Timestep, Region, double> read;
    property leavecost: IVariable2Dimensional<Timestep, Region, double> read;
  end; 
  
  FundOld2ImpactAggregation = public class
  private
  protected
  public
    method Run(clock: Clock; state: IFundOld2ImpactAggregation; legacyData: LegacyFund28Data);    
    
  end;
  
implementation

method FundOld2ImpactAggregation.Run(clock: Clock; state: IFundOld2ImpactAggregation; legacyData: LegacyFund28Data); 
begin
  // Couple SocioEconomic component to FUND 2.8
  // Move data to SocioEconomic component
  for each r: Fund.CommonDimensions.Region in Enum.GetValues(typeof(Fund.CommonDimensions.Region)) do
  begin
    state.forests[clock.Current,r]:=legacyData.economy[ord(r)].forests;
    state.drycost[clock.Current,r]:=legacyData.economy[ord(r)].drycost;
    state.protcost[clock.Current,r]:=legacyData.economy[ord(r)].protcost;
    state.entercost[clock.Current,r]:=legacyData.economy[ord(r)].entercost;
    state.hurrdam[clock.Current,r]:=legacyData.economy[ord(r)].hurrdam;
    state.deadcost[clock.Current,r]:=legacyData.economy[ord(r)].deadcost;
    state.morbcost[clock.Current,r]:=legacyData.economy[ord(r)].morbcost;
    state.wetcost[clock.Current,r]:=legacyData.economy[ord(r)].wetcost;
    state.leavecost[clock.Current,r]:=legacyData.economy[ord(r)].leavecost;
  end; 
end;

end.