// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

uses Esmf;

type
  
  IFundOld2SocioEconomic = public interface
    property leave: IVariable2Dimensional<Timestep, Fund.CommonDimensions.Region, double> read;
    property forcost: IVariable2Dimensional<Timestep, Fund.CommonDimensions.Region, double> read;
    property enter: IVariable2Dimensional<Timestep, Fund.CommonDimensions.Region, double> read;
    property dead: IVariable2Dimensional<Timestep, Fund.CommonDimensions.Region, double> read;
    property forestry: IVariable2Dimensional<Timestep, Fund.CommonDimensions.Region, double> read;
  end; 
  
  FundOld2SocioEconomicComponent = public class
  private
  protected
  public
    method Run(clock: Clock; state: IFundOld2SocioEconomic; legacyData: LegacyFund28Data);    
  end;
  
implementation

method FundOld2SocioEconomicComponent.Run(clock: Clock; state: IFundOld2SocioEconomic; legacyData: LegacyFund28Data); 
begin
  // Couple SocioEconomic component to FUND 2.8
  // Move data to SocioEconomic component
  for each r: Fund.CommonDimensions.Region in Enum.GetValues(typeof(Fund.CommonDimensions.Region)) do
  begin
    state.leave[clock.Current-1,r]:=legacyData.economy[ord(r)].leave/1000000;
    state.forcost[clock.Current-1,r]:=legacyData.economy[ord(r)].forcost;
    state.enter[clock.Current-1,r]:=legacyData.economy[ord(r)].enter/1000000;
    state.dead[clock.Current-1,r]:=legacyData.economy[ord(r)].dead/1000000;
    state.forestry[clock.Current-1,r]:=legacyData.economy[ord(r)].forestry[clock.Current.Value];
  end; 
  
end;

end.