// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

uses
  Esmf;

type
  FundWorkflow = public class
  private
    _modelFields: Esmf.ModelFields:=new ModelFields;
    _legacyData: LegacyFund28Data;
    _parameters: FundParameter;
    
  protected
    method ReCreateStateVariables;
    method InitVariables(stateinterface: System.Type; componentName: string);
    
    method InitClimateState(parameters: FundParameter);
    method InitClimateRegionalState(parameters: FundParameter);
    method InitSocioEconomicState(parameters: FundParameter);
    method InitGlobalWelfareState(parameters: FundParameter);
    method InitRegionalWelfareState(parameters: FundParameter);
    method InitOceanState(parameters: FundParameter);
    method InitBioDiversityState(parameters: FundParameter);
    method InitImpactBioDiversityState(parameters: FundParameter);
    method InitImpactHeatingState(parameters: FundParameter);
    method InitImpactAgricultureState(parameters: FundParameter);
    method InitImpactCoolingState(parameters: FundParameter);
    method InitSocioEconomic2ClimateState(parameters: FundParameter);
    method InitSocioEconomic2ImpactBioDiversityState(parameters: FundParameter);
    method InitFundOld2SocioEconomicState(parameters: FundParameter);
    method InitFundOld2ClimateState(parameters: FundParameter);
    method InitFundOld2GlobalWelfare(parameters: FundParameter);
    method InitSocioEconomic2RegionalWelfare(parameters: FundParameter);
    method InitScenarioUncertaintyState(parameters: FundParameter);
    method InitFundOld2ImpactHeatingCoolingState(parameters: FundParameter);
    method InitImpactAggregationState(parameters: FundParameter);
    method InitFundOld2ImpactAggregationState(parameters: FundParameter);
    method InitGlobalHotellingTaxationState(parameters: FundParameter);
    method InitImpactWaterResourcesState(parameters: FundParameter);
    method InitImpactDiarrhoeaState(parameters: FundParameter);
    
    method RunScenarioUncertaintyComponent(clock: Clock);
    method RunSocioEconomicComponent(clock: Clock);
    method RunGlobalWelfareComponent(clock: Clock);
    method RunRegionalWelfareComponent(clock: Clock);
    method RunClimateComponent(clock: Clock);
    method RunClimateRegionalComponent(clock: Clock);
    method RunOceanComponent(clock: Clock);
    method RunBioDiversityComponent(clock: Clock);
    method RunImpactBioDiversityComponent(clock: Clock);
    method RunImpactHeatingComponent(clock: Clock);
    method RunImpactAgricultureComponent(clock: Clock);
    method RunImpactCoolingComponent(clock: Clock);
    method RunSocioEconomic2ClimateComponent(clock: Clock);
    method RunSocioEconomic2ImpactBioDiversity(clock: Clock);
    method RunFundOld2SocioEconomic(clock: Clock);
    method RunFundOld2ClimateState(clock: Clock);
    method RunFundOld2GlobalWelfareState(clock: Clock);
    method RunSocioEconomic2RegionalWelfareState(clock: Clock);
    method RunFundOld2ImpactHeatingCoolingState(clock: Clock);
    method RunImpactAggregationComponent(clock: Clock);
    method RunFundOld2ImpactAggregationComponent(clock: Clock);
    method RunGlobalHotellingTaxationComponent(clock: Clock);
    method RunImpactWaterResourcesComponent(clock: Clock);
    method RunImpactDiarrhoeaComponent(clock: Clock);
  public
    constructor(parameters: FundParameter);
    method Run(output: ModelOutput);
    
    SocioEconomicState: Fund.Components.SocioEconomic.ISocioEconomicState;
    WelfareState: Fund.Components.Welfare.IGlobalWelfareState; 
    RegionalWelfareState: Fund.Components.Welfare.IRegionalWelfareState;   
    ImpactAggregationState: Fund.Components.ImpactAggregation.IImpactAggregationState;    
    ClimateState: Fund.Components.Climate.IClimateState;
    ClimateRegionalState: Fund.Components.ClimateRegional.IClimateRegionalState;
    OceanState: Fund.Components.Ocean.IOceanState;
    BioDiversityState: Fund.Components.BioDiversity.IBioDiversityState;
    ImpactBioDiversityState: Fund.Components.ImpactBioDiversity.IImpactBioDiversityState;
    SocioEconomic2ClimateState: Fund.ISocioEconomic2Climate;
    SocioEconomic2ImpactBioDiversityState: Fund.ISocioEconomic2ImpactBioDiversityState;
    FundOld2SocioEconomicState: Fund.IFundOld2SocioEconomic;
    FundOld2ClimateState: Fund.IFundOld2Climate;
    FundOld2GlobalWelfareState: Fund.ISocioEconomic2Welfare;
    SocioEconomic2RegionalWelfareState: Fund.ISocioEconomic2RegionalWelfare;
    ScenarioUncertaintyState: Fund.Components.ScenarioUncertainty.IScenarioUncertaintyState;
    ImpactHeatingState: Fund.Components.ImpactHeating.IImpactHeatingState;
    ImpactCoolingState: Fund.Components.ImpactCooling.IImpactCoolingState;
    FundOld2ImpactHeatingCoolingState: Fund.IFundOld2ImpactHeatingCoolingState;
    ImpactAgricultureState: Fund.Components.ImpactAgriculture.IImpactAgricultureState;
    FundOld2ImpactAggregationState: Fund.IFundOld2ImpactAggregation;
    GlobalHotellingTaxationState: Fund.Components.HotellingTaxation.IGlobalHotellingTaxationState;
    ImpactWaterResourcesState: Fund.Components.ImpactWaterResources.IImpactWaterResourcesState;
    ImpactDiarrhoeaState: Fund.Components.ImpactDiarrhoea.IImpactDiarrhoeaState;
    
    property ModelFields: ModelFields read _modelFields;
  end;
  
implementation

procedure FundWorkflow.Run(output: ModelOutput);
var
  clock: Clock;
begin
  ReCreateStateVariables;
  
  clock:=new Clock(Timestep.FromSimulationYear(0),Timestep.FromSimulationYear(LegacyConstants.NYear));
    
  RunScenarioUncertaintyComponent(clock);
  clock.Advance;
     
  while not clock.IsDone do
  begin
    RunScenarioUncertaintyComponent(clock);
    
    RunFundOld2SocioEconomic(clock);    
    RunSocioEconomicComponent(clock);
    
    RunGlobalHotellingTaxationComponent(clock);
    
    RunSocioEconomic2ClimateComponent(clock);
    RunFundOld2ClimateState(clock);    
    RunClimateComponent(clock);
    RunClimateRegionalComponent(clock);
  
    RunOceanComponent(clock);
  
    RunBioDiversityComponent(clock);
    
    RunSocioEconomic2ImpactBioDiversity(clock);
    RunImpactBioDiversityComponent(clock);
    
    RunFundOld2ImpactHeatingCoolingState(clock);
    RunImpactHeatingComponent(clock);
    RunImpactCoolingComponent(clock);
    
    RunImpactAgricultureComponent(clock);

    RunImpactWaterResourcesComponent(clock);

    RunImpactDiarrhoeaComponent(clock);

    for each r: Fund.CommonDimensions.Region in Enum.GetValues(typeof(Fund.CommonDimensions.Region)) do
    begin
      _legacyData.Calculate(var _legacyData.economy[ord(r)],clock.Current.Value, output);
      
      if Output<>nil then
      begin
        // Add GDP for that year and region to the Output object
        // income is multiplied by 1 billion, since that is the unit
        // of income
        output.Incomes.Add(clock.Current.Value, ord(r), _legacyData.economy[ord(r)].income * 1000000000);

        // Add population for that year and region to the Output object
        // population is multiplied by 1 million, since that is the unit of
        // population
        output.Populations.Add(clock.Current.Value, ord(r), _legacyData.economy[ord(r)].population * 1000000);
      end;  
      
    end;
    
    RunFundOld2ImpactAggregationComponent(clock);
    RunImpactAggregationComponent(clock);
    
    RunFundOld2GlobalWelfareState(clock);
    RunGlobalWelfareComponent(clock);

    RunSocioEconomic2RegionalWelfareState(clock);
    RunRegionalWelfareComponent(clock);
    
    clock.Advance;         
  end;  
end;

method FundWorkflow.RunSocioEconomicComponent(clock: Clock);
begin
  // Set up SocioEconomic component
  var lSocioEconomicComponent:=new Fund.Components.SocioEconomic.SocioEconomicComponent;  
          
  // Run SocioEconomic component
  lSocioEconomicComponent.Run(clock, SocioEconomicState);
          
  // Couple SocioEconomic component fo FUND 2.8
  // Move data out of SocioEconomic component
  for each r: Fund.CommonDimensions.Region in Enum.GetValues(typeof(Fund.CommonDimensions.Region)) do
  begin
    _legacyData.economy[ord(r)].income:=SocioEconomicState.income[Clock.Current,r];
    _legacyData.economy[ord(r)].oldincome:=SocioEconomicState.income[Clock.Current-1,r];
    _legacyData.economy[ord(r)].population:=SocioEconomicState.population[Clock.Current, r];
    _legacyData.economy[ord(r)].emission:=SocioEconomicState.emission[Clock.Current, r];
    _legacyData.economy[ord(r)].reei:=SocioEconomicState.reei[Clock.Current,r];
    _legacyData.economy[ord(r)].seei:=SocioEconomicState.seei[Clock.Current,r];
  end; 
end;


method FundWorkflow.RunClimateComponent(clock: Clock); 
begin       
  var t:=clock.Current;
                        
  // Set up climate component
  var lclimateComp:=new Fund.Components.Climate.ClimateComponent;
            
  // Run climate component
  lclimateComp.Run(clock, ClimateState);
          
  // Couple climate component fo FUND 2.8
  // Move data out of climate component
  _legacyData.Climate.acco2[t.Value]:=ClimateState.acco2[t];
end;

method FundWorkflow.InitClimateState(parameters: FundParameter); 
begin  
  // Constant parameters
  _modelFields.AddNonDimensionalVariable<double>('climate','temps',parameters.temps);
  _modelFields.AddNonDimensionalVariable<double>('climate','so2ind',0.08);
  _modelFields.AddNonDimensionalVariable<double>('climate','so2dir',0.03);
  _modelFields.AddNonDimensionalVariable<double>('climate','sf6pre',0.04);
  _modelFields.AddNonDimensionalVariable<double>('climate','sf6decay',parameters.sf6decay);
  _modelFields.AddNonDimensionalVariable<double>('climate','n2opre',285);
  _modelFields.AddNonDimensionalVariable<double>('climate','n2odecay',parameters.n2odecay);
  _modelFields.AddNonDimensionalVariable<double>('climate','delaytemp',parameters.delaytemp);
  _modelFields.AddNonDimensionalVariable<double>('climate','co2pre',275);
  _modelFields.AddNonDimensionalVariable<double>('climate','co2frac5',0.10);
  _modelFields.AddNonDimensionalVariable<double>('climate','co2frac4',0.25);
  _modelFields.AddNonDimensionalVariable<double>('climate','co2frac3',0.32);
  _modelFields.AddNonDimensionalVariable<double>('climate','co2frac2',0.20);
  _modelFields.AddNonDimensionalVariable<double>('climate','co2frac1',0.13);
  _modelFields.AddNonDimensionalVariable<double>('climate','co2decay5',parameters.decay[5]);
  _modelFields.AddNonDimensionalVariable<double>('climate','co2decay4',parameters.decay[4]);
  _modelFields.AddNonDimensionalVariable<double>('climate','co2decay3',parameters.decay[3]);
  _modelFields.AddNonDimensionalVariable<double>('climate','co2decay2',parameters.decay[2]);
  _modelFields.AddNonDimensionalVariable<double>('climate','co2decay1',parameters.decay[1]);
  _modelFields.AddNonDimensionalVariable<double>('climate','ch4pre',790);
  _modelFields.AddNonDimensionalVariable<double>('climate','ch4decay',parameters.ch4decay);
  _modelFields.Add1DimensionalParameter<Esmf.Timestep,double>('climate','RunAwayCH4', new YearParameterConnector(parameters, 'RunAwayCH4'));    
  _modelFields.AddNonDimensionalVariable<double>('climate','ch4add',parameters.ch4add);
  _modelFields.AddNonDimensionalVariable<double>('climate','n2oadd',parameters.n2oadd);
  _modelFields.AddNonDimensionalVariable<double>('climate','sf6add',parameters.sf6add);
  _modelFields.AddNonDimensionalVariable<boolean>('climate','addmp',false);  
  
  // Parameters coming from different components, need to be coupled
  _modelFields.ConnectParameterToVariable('climate','mco2','socioeconomic2climate','mco2');  
  _modelFields.ConnectParameterToVariable('climate','globso2','socioeconomic2climate','globso2');
  _modelFields.ConnectParameterToVariable('climate','globsf6','socioeconomic2climate','globsf6');
  _modelFields.ConnectParameterToVariable('climate','globn2o','socioeconomic2climate','globn2o');
  _modelFields.ConnectParameterToVariable('climate','globch4','socioeconomic2climate','globch4');  
  _modelFields.ConnectParameterToVariable('climate', 'addco2', 'fundold2climate', 'addco2');
  _modelFields.ConnectParameterToVariable('climate', 'addch4', 'fundold2climate', 'addch4');
  _modelFields.ConnectParameterToVariable('climate', 'addn2o', 'fundold2climate', 'addn2o');
  _modelFields.ConnectParameterToVariable('climate', 'addsf6', 'fundold2climate', 'addsf6');
    
  // Create proxy
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Climate.IClimateState));
  ClimateState:=ss.ConnectToState<Fund.Components.Climate.IClimateState>(_modelFields, 'climate');
  
  // Initial values, should eventually also come from parameter file
  ClimateState.temp[Timestep.FromSimulationYear(0)]:= 0.20;
  ClimateState.cbox1[Timestep.FromSimulationYear(0)]:=283.53;
  ClimateState.cbox2[Timestep.FromSimulationYear(0)]:=5.62;
  ClimateState.cbox3[Timestep.FromSimulationYear(0)]:=6.29;
  ClimateState.cbox4[Timestep.FromSimulationYear(0)]:=2.19;
  ClimateState.cbox5[Timestep.FromSimulationYear(0)]:=0.15;
  ClimateState.acsf6[Timestep.FromSimulationYear(0)]:=ClimateState.sf6pre;
  ClimateState.acn2o[Timestep.FromSimulationYear(0)]:=296;
  ClimateState.acch4[Timestep.FromSimulationYear(0)]:=1222;
  ClimateState.maxrf:=0;
end;

method FundWorkflow.InitSocioEconomicState(parameters: FundParameter);
begin   
  // Constant parameters
  _modelFields.AddNonDimensionalVariable<double>('socioeconomic','consleak',0.25);
  _modelFields.AddNonDimensionalVariable<double>('socioeconomic','so2pop',0.33);
  _modelFields.AddNonDimensionalVariable<double>('socioeconomic','so2inc',-0.45);  
  _modelFields.AddNonDimensionalVariable<double>('socioeconomic','so2carb',1.02);
  _modelFields.AddNonDimensionalVariable<double>('socioeconomic','sf6gdp',parameters.sf6gdp);
  _modelFields.AddNonDimensionalVariable<double>('socioeconomic','sf6ypc',parameters.sf6ypc);
  _modelFields.AddNonDimensionalVariable<double>('socioeconomic','knowpar',0.9);
  _modelFields.AddNonDimensionalVariable<double>('socioeconomic','knowgpar',0.1);
  _modelFields.AddNonDimensionalVariable<double>('socioeconomic','gwpch4',21.0*22.0/6.0/1000);
  _modelFields.AddNonDimensionalVariable<double>('socioeconomic','gwpn2o',310.0*7.0/6.0/1000);
  _modelFields.AddNonDimensionalVariable<boolean>('socioeconomic', 'runwithoutdamage', false);
  _modelFields.AddNonDimensionalVariable<boolean>('socioeconomic', 'runwithoutpopulationperturbation', false);
  
  // Parameters coming from different components, directly coupled
  _modelFields.ConnectParameterToVariable('socioeconomic', 'ypcgrowth', 'scenariouncertainty', 'ypcgrowth');  
  _modelFields.ConnectParameterToVariable('socioeconomic', 'pgrowth', 'scenariouncertainty', 'pgrowth'); 
  _modelFields.ConnectParameterToVariable('socioeconomic', 'currtax', 'globalhotellingtaxation', 'currtax');  
    
  // Parameters coming from different components, need to be coupled
  _modelFields.ConnectParameterToVariable('socioeconomic','leave','fundold2socioeconomic','leave');  
  _modelFields.ConnectParameterToVariable('socioeconomic','forcost','fundold2socioeconomic','forcost');  
  _modelFields.ConnectParameterToVariable('socioeconomic','enter','fundold2socioeconomic','enter');  
  _modelFields.ConnectParameterToVariable('socioeconomic','eloss','impactaggregation','eloss');
  _modelFields.ConnectParameterToVariable('socioeconomic','sloss','impactaggregation','sloss');
  _modelFields.ConnectParameterToVariable('socioeconomic','dead','fundold2socioeconomic','dead');  
  _modelFields.ConnectParameterToVariable('socioeconomic','forestry','fundold2socioeconomic','forestry');  
      
  // Parameters coming from file
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('socioeconomic','taxmp', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'taxmp'));
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('socioeconomic','gdp0', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'GDP0'));
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('socioeconomic','formp', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'formp'));
  _modelFields.Add2DimensionalParameter<Timestep,Fund.CommonDimensions.Region,double>('socioeconomic','aeei', new YearRegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'AEEI'));
  _modelFields.Add2DimensionalParameter<Timestep,Fund.CommonDimensions.Region,double>('socioeconomic','acei', new YearRegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'ACEI'));
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('socioeconomic','sf60', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'sf60'));
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('socioeconomic','GDP90', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'gdp90'));
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('socioeconomic','pop90', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'pop90'));
  _modelFields.Add2DimensionalParameter<Timestep,Fund.CommonDimensions.Region,double>('socioeconomic','ch4em', new YearRegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'ch4em')); 
  _modelFields.Add2DimensionalParameter<Timestep,Fund.CommonDimensions.Region,double>('socioeconomic','n2oem', new YearRegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'n2oem')); 
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('socioeconomic','ch4par1', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'ch4par1'));  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('socioeconomic','ch4par2', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'ch4par2'));  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('socioeconomic','n2opar1', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'n2opar1'));  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('socioeconomic','n2opar2', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'n2opar2'));  
  _modelFields.Add2DimensionalParameter<Timestep,Fund.CommonDimensions.Region,double>('socioeconomic','forestemm', new YearRegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'ForestEmm')); 
  
  // Create proxy
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.SocioEconomic.ISocioEconomicState));
  SocioEconomicState:=ss.ConnectToState<Fund.Components.SocioEconomic.ISocioEconomicState>(_modelFields, 'socioeconomic');
  
  // Initial values, should eventually also come from parameter file
  var t0:=Timestep.FromSimulationYear(0);
  for each r: Fund.CommonDimensions.Region in Enum.GetValues(typeof(Fund.CommonDimensions.Region)) do
  begin
    SocioEconomicState.income[t0, r]:=parameters.gdp0[ord(r)];
    SocioEconomicState.population[t0, r]:=parameters.pop0[ord(r)];
    SocioEconomicState.energint[t0, r]:=1;
    SocioEconomicState.energuse[t0, r]:=_parameters.gdp0[ord(r)];
    SocioEconomicState.emissint[t0, r]:=_parameters.emissint0[ord(r)];
    SocioEconomicState.so2[t0, r]:=_parameters.so20[ord(r)];
    SocioEconomicState.emission[t0, r]:=SocioEconomicState.emissint[t0,r]/SocioEconomicState.energuse[t0,r];
    SocioEconomicState.ch4cost[t0, r]:=0;
    SocioEconomicState.n2ocost[t0,r]:=0;
    SocioEconomicState.ryg[t0,r]:=0;
    SocioEconomicState.reei[t0,r]:=0;
    SocioEconomicState.rcei[t0,r]:=0;
    SocioEconomicState.seei[t0,r]:=0;
    SocioEconomicState.scei[t0,r]:=0;
    SocioEconomicState.co2red[t0,r]:=0;
    SocioEconomicState.know[t0,r]:=1;
    SocioEconomicState.ch4red[t0,r]:=0;
    SocioEconomicState.n2ored[t0,r]:=0;
  end;
  SocioEconomicState.globknow[t0]:=1;
  
  //SocioEconomicState.minint[t0]:=Double.PositiveInfinity;
  var minint:=double.PositiveInfinity;
  for each r: Fund.CommonDimensions.Region in Enum.GetValues(typeof(Fund.CommonDimensions.Region)) do
  begin              
    if SocioEconomicState.emission[t0,r]/SocioEconomicState.income[t0,r] < minint then
      minint:=SocioEconomicState.emission[t0,r]/SocioEconomicState.income[t0,r];
  end;
  SocioEconomicState.minint[t0]:=minint;
  
end;

constructor FundWorkflow(parameters: FundParameter);
begin
  _parameters:=parameters;
  _legacyData:=new LegacyFund28Data(_parameters);
  
  InitVariables(typeof(Fund.Components.ScenarioUncertainty.IScenarioUncertaintyState), 'scenariouncertainty');
  InitVariables(typeof(Fund.Components.BioDiversity.IBioDiversityState), 'biodiversity');
  InitVariables(typeof(Fund.Components.Climate.IClimateState), 'climate');
  InitVariables(typeof(Fund.Components.ClimateRegional.IClimateRegionalState), 'climateregional');
  InitVariables(typeof(Fund.Components.ImpactBioDiversity.IImpactBioDiversityState), 'impactbiodiversity');
  InitVariables(typeof(Fund.Components.ImpactHeating.IImpactHeatingState), 'impactheating');
  InitVariables(typeof(Fund.Components.ImpactAgriculture.IImpactAgricultureState), 'impactagriculture');
  InitVariables(typeof(Fund.Components.ImpactCooling.IImpactCoolingState), 'impactcooling');
  InitVariables(typeof(Fund.Components.Ocean.IOceanState), 'ocean');
  InitVariables(typeof(Fund.Components.SocioEconomic.ISocioEconomicState), 'socioeconomic');
  InitVariables(typeof(Fund.Components.Welfare.IGlobalWelfareState), 'globalwelfare');
  InitVariables(typeof(Fund.Components.Welfare.IRegionalWelfareState), 'regionalwelfare');
  InitVariables(typeof(Fund.ISocioEconomic2Climate), 'socioeconomic2climate');
  InitVariables(typeof(Fund.IFundOld2SocioEconomic), 'fundold2socioeconomic');
  InitVariables(typeof(Fund.IFundOld2Climate), 'fundold2climate');
  InitVariables(typeof(Fund.ISocioEconomic2ImpactBioDiversityState), 'socioeconomic2impactbiodiversity');
  InitVariables(typeof(Fund.ISocioEconomic2Welfare), 'fundold2globalwelfare');
  InitVariables(typeof(Fund.ISocioEconomic2RegionalWelfare), 'socioeconomic2regionalwelfare');
  InitVariables(typeof(Fund.IFundOld2ImpactHeatingCoolingState), 'fundold2impactheatingcooling');
  InitVariables(typeof(Fund.Components.ImpactAggregation.IImpactAggregationState), 'impactaggregation');
  InitVariables(typeof(Fund.IFundOld2ImpactAggregation), 'fundold2impactaggregation');
  InitVariables(typeof(Fund.Components.HotellingTaxation.IGlobalHotellingTaxationState), 'globalhotellingtaxation');
  InitVariables(typeOf(Fund.Components.ImpactWaterResources.IImpactWaterResourcesState), 'impactwaterresources');
  InitVariables(typeOf(Fund.Components.ImpactDiarrhoea.IImpactDiarrhoeaState), 'impactdiarrhoea');
  
  InitScenarioUncertaintyState(_parameters);
  InitClimateState(_parameters);
  InitClimateRegionalState(_parameters);
  InitOceanState(_parameters);
  InitBioDiversityState(_parameters);
  InitSocioEconomicState(_parameters);
  InitGlobalWelfareState(_parameters);
  InitRegionalWelfareState(_parameters);
  InitImpactBioDiversityState(_parameters);
  InitImpactHeatingState(_parameters);
  InitImpactAgricultureState(_parameters);
  InitImpactCoolingState(_parameters);
  InitFundOld2SocioEconomicState(_parameters);
  InitSocioEconomic2ClimateState(_parameters);
  InitFundOld2ClimateState(_parameters);
  InitSocioEconomic2ImpactBioDiversityState(_parameters);
  InitFundOld2GlobalWelfare(_parameters);
  InitSocioEconomic2RegionalWelfare(_parameters);
  InitFundOld2ImpactHeatingCoolingState(_parameters);
  InitImpactAggregationState(_parameters);
  InitFundOld2ImpactAggregationState(_parameters);
  InitGlobalHotellingTaxationState(_parameters);
  InitImpactWaterResourcesState(_parameters);
  InitImpactDiarrhoeaState(_parameters);
end;

method FundWorkflow.InitOceanState(parameters: FundParameter); 
begin   
  // Constant parameters
  _modelFields.AddNonDimensionalVariable<double>('ocean','delaysea',parameters.delaysea);
  _modelFields.AddNonDimensionalVariable<double>('ocean','seas',parameters.seas);
  
  // Parameters coming from different components, directly coupled
  _modelFields.ConnectParameterToVariable('ocean', 'temp', 'climate', 'temp');
  
  // Create proxy
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Ocean.IOceanState));
  OceanState:=ss.ConnectToState<Fund.Components.Ocean.IOceanState>(_modelFields, 'ocean');
  
  // Initial values, should eventually also come from parameter file
  var t0:=Timestep.FromSimulationYear(0);
  OceanState.sea[t0]:=0;
end;

method FundWorkflow.RunOceanComponent(clock: Clock); 
begin
  var t:=clock.Current;
                       
  // Set up ocean component
  var oc:=new Fund.Components.Ocean.OceanComponent;
            
  // Run ocean component
  oc.Run(clock, OceanState);
          
  // Couple ocean component to FUND 2.8
  // Move data out of ocean component
  _legacyData.Climate.sea:=OceanState.sea[t];
  _legacyData.Climate.ds:=OceanState.sea[t]-OceanState.sea[t-1];
end;

method FundWorkflow.InitBioDiversityState(parameters: FundParameter); 
begin
  // Constant parameters
  _modelFields.AddNonDimensionalVariable<double>('biodiversity','nospecbase',parameters.nospecbase);
  _modelFields.AddNonDimensionalVariable<double>('biodiversity','bioloss',parameters.bioloss);
  _modelFields.AddNonDimensionalVariable<double>('biodiversity','biosens',parameters.biosens);
  _modelFields.AddNonDimensionalVariable<double>('biodiversity','dbsta',parameters.DBsTa);
  
  // Parameters coming from different components, directly coupled
  _modelFields.ConnectParameterToVariable('biodiversity','temp','climate','temp');

  // Create proxy
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.BioDiversity.IBioDiversityState));
  BioDiversityState:=ss.ConnectToState<Fund.Components.BioDiversity.IBioDiversityState>(_modelFields, 'biodiversity');
  
  // Initial values, should eventually also come from parameter file
  // Carefull, the initial value is for the year 50!
  BioDiversityState.nospecies[Timestep.FromSimulationYear(0)]:=BioDiversityState.nospecbase;
end;

method FundWorkflow.RunBioDiversityComponent(clock: Clock); 
begin                      
  // Set up biodiv component
  var bc:=new Fund.Components.BioDiversity.BioDiversityComponent;
            
  // Run biodiv component
  bc.Run(clock, BioDiversityState);
end;

method FundWorkflow.InitImpactBioDiversityState(parameters: FundParameter); 
begin
  // Constant parameters
  _modelFields.AddNonDimensionalVariable<double>('impactbiodiversity','nospecbase',parameters.nospecbase);
  _modelFields.AddNonDimensionalVariable<double>('impactbiodiversity','bioshare',parameters.bioshare);
  _modelFields.AddNonDimensionalVariable<double>('impactbiodiversity','spbm',parameters.spbm);
  _modelFields.AddNonDimensionalVariable<double>('impactbiodiversity','valbase',parameters.valbase);
  _modelFields.AddNonDimensionalVariable<double>('impactbiodiversity','dbsta',parameters.DBsTa);
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactbiodiversity','valinc', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'valinc'));  
  
  // Parameters coming from different components, directly coupled
  _modelFields.ConnectParameterToVariable('impactbiodiversity', 'temp', 'climate', 'temp');  
  _modelFields.ConnectParameterToVariable('impactbiodiversity', 'nospecies', 'biodiversity', 'nospecies');  
  
  // Parameters coming from different components, explicitly coupled
  _modelFields.ConnectParameterToVariable('impactbiodiversity', 'income', 'socioeconomic2impactbiodiversity', 'income');  
  _modelFields.ConnectParameterToVariable('impactbiodiversity', 'population', 'socioeconomic2impactbiodiversity', 'population');    
  
  // Create proxy
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactBioDiversity.IImpactBioDiversityState));
  ImpactBioDiversityState:=ss.ConnectToState<Fund.Components.ImpactBioDiversity.IImpactBioDiversityState>(_modelFields, 'impactbiodiversity');
  
end;

method FundWorkflow.RunImpactBioDiversityComponent(clock: Clock); 
begin
  // Set up ImpactBioDiversity component
  var lImpactBioDiversityComponent:=new Fund.Components.ImpactBioDiversity.ImpactBioDiversityComponent;
            
  // Run ImpactBioDiversity component
  lImpactBioDiversityComponent.Run(clock, ImpactBioDiversityState);
          
  // Couple ImpactBioDiversity component fo FUND 2.8
  // Move data out of ImpactBioDiversity component
  for each r: Fund.CommonDimensions.Region in Enum.GetValues(typeof(Fund.CommonDimensions.Region)) do
  begin
    _legacyData.economy[ord(r)].species:=ImpactBioDiversityState.species[Clock.Current,r];
  end; 
end;

method FundWorkflow.InitVariables(stateinterface: System.Type; componentName: string);
begin
  // Create proxy
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(stateinterface);
  ss.CreateVariables(_modelFields, componentName);
end;
  
method FundWorkflow.RunSocioEconomic2ClimateComponent(clock: Clock);
begin
  var c:=new Fund.SocioEconomic2Climate;
            
  c.Run(clock, SocioEconomic2ClimateState);
end;

method FundWorkflow.RunFundOld2SocioEconomic(clock: Clock);
begin
  var c:=new Fund.FundOld2SocioEconomicComponent;
            
  c.Run(clock, FundOld2SocioEconomicState, _legacyData);
end;

method FundWorkflow.InitFundOld2SocioEconomicState(parameters: FundParameter);
begin
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.IFundOld2SocioEconomic));
  FundOld2SocioEconomicState:=ss.ConnectToState<Fund.IFundOld2SocioEconomic>(_modelFields, 'fundold2socioeconomic');
end;

method FundWorkflow.InitSocioEconomic2ClimateState(parameters: FundParameter); 
begin
  // Parameters coming from different components, directly coupled
  _modelFields.ConnectParameterToVariable('socioeconomic2climate', 'emissionwithforestry', 'socioeconomic', 'emissionwithforestry');  
  _modelFields.ConnectParameterToVariable('socioeconomic2climate', 'ch4', 'socioeconomic', 'ch4');  
  _modelFields.ConnectParameterToVariable('socioeconomic2climate', 'n2o', 'socioeconomic', 'n2o');  
  _modelFields.ConnectParameterToVariable('socioeconomic2climate', 'sf6', 'socioeconomic', 'sf6');  
  _modelFields.ConnectParameterToVariable('socioeconomic2climate', 'so2withseeiandscei', 'socioeconomic', 'so2WithSeeiAndScei');  
  
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.ISocioEconomic2Climate));
  SocioEconomic2ClimateState:=ss.ConnectToState<Fund.ISocioEconomic2Climate>(_modelFields, 'socioeconomic2climate');  
end;

method FundWorkflow.InitFundOld2ClimateState(parameters: FundParameter); 
begin
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.IFundOld2Climate));
  FundOld2ClimateState:=ss.ConnectToState<Fund.IFundOld2Climate>(_modelFields, 'fundold2climate');  
end;

method FundWorkflow.RunFundOld2ClimateState(clock: Clock); 
begin
  var c:=new Fund.FundOld2ClimateComponent;
            
  c.Run(clock, FundOld2ClimateState,_parameters);  
end;

method FundWorkflow.InitSocioEconomic2ImpactBioDiversityState(parameters: FundParameter); 
begin
  // Parameters coming from different components, directly coupled
  _modelFields.ConnectParameterToVariable('socioeconomic2impactbiodiversity', 'incomesource', 'socioeconomic', 'income');  
  _modelFields.ConnectParameterToVariable('socioeconomic2impactbiodiversity', 'populationsource', 'socioeconomic', 'population');  
  
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.ISocioEconomic2ImpactBioDiversityState));
  SocioEconomic2ImpactBioDiversityState:=ss.ConnectToState<Fund.ISocioEconomic2ImpactBioDiversityState>(_modelFields, 'socioeconomic2impactbiodiversity');  
end;

method FundWorkflow.RunSocioEconomic2ImpactBioDiversity(clock: Clock); 
begin
  var c:=new Fund.SocioEconomic2ImpactBioDiversity;
            
  c.Run(clock, SocioEconomic2ImpactBioDiversityState);    
end;

method FundWorkflow.InitGlobalWelfareState(parameters: FundParameter); 
begin   
  // Constant parameters
  _modelFields.AddNonDimensionalVariable<double>('globalwelfare', 'prtp', 0.03);
  _modelFields.AddNonDimensionalVariable<double>('globalwelfare', 'elasticityofmarginalutility', 1.0);
  _modelFields.AddNonDimensionalVariable<double>('globalwelfare', 'utilitycalibrationadditive', 0.0);
  _modelFields.AddNonDimensionalVariable<double>('globalwelfare', 'utilitycalibrationmultiplicative', 1.0);
  
  // Parameters coming from different components, need to be coupled
  _modelFields.ConnectParameterToVariable('globalwelfare','consumption','fundold2globalwelfare','totalconsumption');  
  _modelFields.ConnectParameterToVariable('globalwelfare','population','fundold2globalwelfare','totalpopulation');  
       
  // Create proxy
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Welfare.IGlobalWelfareState));
  WelfareState:=ss.ConnectToState<Fund.Components.Welfare.IGlobalWelfareState>(_modelFields, 'globalwelfare');
  
  // Initial values, should eventually also come from parameter file
  var t0:=Timestep.FromSimulationYear(0);
  //WelfareState.welfare[t0]:=0;
end;

method FundWorkflow.InitFundOld2GlobalWelfare(parameters: FundParameter); 
begin
  // Parameters coming from different components, need to be coupled
  _modelFields.ConnectParameterToVariable('fundold2globalwelfare','consumption','socioeconomic','consumption');  
  _modelFields.ConnectParameterToVariable('fundold2globalwelfare','population','socioeconomic','population');  
  
  
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.ISocioEconomic2Welfare));
  FundOld2GlobalWelfareState:=ss.ConnectToState<Fund.ISocioEconomic2Welfare>(_modelFields, 'fundold2globalwelfare');  
end;

method FundWorkflow.RunFundOld2GlobalWelfareState(clock: Clock); 
begin
  var c:=new Fund.SocioEconomic2Welfare;
            
  c.Run(clock, FundOld2GlobalWelfareState);  
end;

method FundWorkflow.RunGlobalWelfareComponent(clock: Clock); 
begin
  // Set up SocioEconomic component
  var lComponent:=new Fund.Components.Welfare.GlobalWelfareComponent;
          
  // Run SocioEconomic component
  lComponent.Run(clock, WelfareState);          
end;

method FundWorkflow.InitRegionalWelfareState(parameters: FundParameter); 
begin
  // Constant parameters
  _modelFields.AddNonDimensionalVariable<double>('regionalwelfare', 'prtp', 0.03);
  _modelFields.AddNonDimensionalVariable<double>('regionalwelfare', 'elasticityofmarginalutility', 1.0);
  _modelFields.AddNonDimensionalVariable<double>('regionalwelfare', 'utilitycalibrationadditive', 0.0);
  _modelFields.AddNonDimensionalVariable<double>('regionalwelfare', 'utilitycalibrationmultiplicative', 1.0);
  _modelFields.AddNonDimensionalVariable<Timestep>('regionalwelfare', 'starttimestep', Timestep.FromYear(2000));
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,Double>('regionalwelfare', 'welfareweight',
    method(r: Fund.CommonDimensions.Region): Double;
    begin
      Result:=1.0;
    end);
  
  // Parameters coming from different components, directly coupled
    _modelFields.ConnectParameterToVariable('regionalwelfare', 'consumption', 'socioeconomic', 'consumption');  
    _modelFields.ConnectParameterToVariable('regionalwelfare', 'income', 'socioeconomic', 'income');
    _modelFields.ConnectParameterToVariable('regionalwelfare', 'loss', 'impactaggregation', 'loss');
  
  // Parameters coming from different components, need to be coupled
  _modelFields.ConnectParameterToVariable('regionalwelfare','population','socioeconomic2regionalwelfare','population');  
       
  // Create proxy
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Welfare.IRegionalWelfareState));
  RegionalWelfareState:=ss.ConnectToState<Fund.Components.Welfare.IRegionalWelfareState>(_modelFields, 'regionalwelfare');
  
  // Initial values, should eventually also come from parameter file
  var t0:=Timestep.FromSimulationYear(0);
  //WelfareState.welfare[t0]:=0;
end;

method FundWorkflow.RunRegionalWelfareComponent(clock: Clock); 
begin
  // Set up SocioEconomic component
  var lComponent:=new Fund.Components.Welfare.RegionalWelfareComponent;
          
  // Run SocioEconomic component
  lComponent.Run(clock, RegionalWelfareState);          
end;

method FundWorkflow.InitSocioEconomic2RegionalWelfare(parameters: FundParameter); 
begin
  // Parameters coming from different components, need to be coupled
  _modelFields.ConnectParameterToVariable('socioeconomic2regionalwelfare','inputpopulation','socioeconomic','population');  
  
  
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.ISocioEconomic2RegionalWelfare));
  SocioEconomic2RegionalWelfareState:=ss.ConnectToState<Fund.ISocioEconomic2RegionalWelfare>(_modelFields, 'socioeconomic2regionalwelfare');  
end;

method FundWorkflow.RunSocioEconomic2RegionalWelfareState(clock: Clock); 
begin
  var c:=new Fund.SocioEconomic2RegionalWelfare;
            
  c.Run(clock, SocioEconomic2RegionalWelfareState);  
end;

method FundWorkflow.InitScenarioUncertaintyState(parameters: FundParameter); 
begin
  // Constant parameters
  _modelFields.AddNonDimensionalVariable<Timestep>('scenariouncertainty','timeofuncertaintystart',Timestep.FromSimulationYear(50));
         
  // Parameters coming from file
  _modelFields.Add2DimensionalParameter<Timestep,Fund.CommonDimensions.Region,double>('scenariouncertainty','inputypcgrowth', new YearRegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'ypcgrowth'));
  _modelFields.Add2DimensionalParameter<Timestep,Fund.CommonDimensions.Region,double>('scenariouncertainty','inputpgrowth', new YearRegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'pgrowth'));
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('scenariouncertainty','ecgradd', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'ecgradd'));  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('scenariouncertainty','pgadd', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'pgadd'));  
  
  // Create proxy
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ScenarioUncertainty.IScenarioUncertaintyState));
  ScenarioUncertaintyState:=ss.ConnectToState<Fund.Components.ScenarioUncertainty.IScenarioUncertaintyState>(_modelFields, 'scenariouncertainty');
end;

method FundWorkflow.RunScenarioUncertaintyComponent(clock: Clock); 
begin
  var c:=new Fund.Components.ScenarioUncertainty.ScenarioUncertaintyComponent;
            
  c.Run(clock, ScenarioUncertaintyState);    
end;

method FundWorkflow.InitImpactHeatingState(parameters: FundParameter); 
begin
  // Constant parameters
  _modelFields.AddNonDimensionalVariable<double>('impactheating','heel',parameters.heel);
  _modelFields.AddNonDimensionalVariable<double>('impactheating','henl',parameters.henl);  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactheating','hebm', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'hebm'));  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactheating','gdp90', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'gdp90'));  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactheating','pop90', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'pop90'));  
  
  // Parameters coming from different components, directly coupled
  _modelFields.ConnectParameterToVariable('impactheating', 'population', 'socioeconomic', 'population');  
  _modelFields.ConnectParameterToVariable('impactheating', 'income', 'socioeconomic', 'income');  
  _modelFields.ConnectParameterToVariable('impactheating', 'temp', 'climate', 'temp');  
  
  // Parameters coming from different components, explicitly coupled
  _modelFields.ConnectParameterToVariable('impactheating', 'cumaeei', 'fundold2impactheatingcooling', 'cumaeei');  
  
  // Create proxy
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactHeating.IImpactHeatingState));
  ImpactHeatingState:=ss.ConnectToState<Fund.Components.ImpactHeating.IImpactHeatingState>(_modelFields, 'impactheating');
end;

method FundWorkflow.RunImpactHeatingComponent(clock: Clock); 
begin
  var c:=new Fund.Components.ImpactHeating.ImpactHeatingComponent;
            
  // Run ImpactBioDiversity component
  c.Run(clock, ImpactHeatingState);
          
  for each r: Fund.CommonDimensions.Region in Enum.GetValues(typeof(Fund.CommonDimensions.Region)) do
  begin
    _legacyData.economy[ord(r)].heating:=ImpactHeatingState.heating[clock.Current,r];
  end; 
end;

method FundWorkflow.InitFundOld2ImpactHeatingCoolingState(parameters: FundParameter); 
begin
  // Create proxy
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.IFundOld2ImpactHeatingCoolingState));
  FundOld2ImpactHeatingCoolingState:=ss.ConnectToState<Fund.IFundOld2ImpactHeatingCoolingState>(_modelFields, 'fundold2impactheatingcooling');
end;

method FundWorkflow.RunFundOld2ImpactHeatingCoolingState(clock: Clock); 
begin
  var c:=new Fund.FundOld2ImpactHeatingCooling;
            
  // Run ImpactBioDiversity component
  c.Run(clock, FundOld2ImpactHeatingCoolingState, _legacyData);          
end;

method FundWorkflow.InitImpactCoolingState(parameters: FundParameter); 
begin
  // Constant parameters
  _modelFields.AddNonDimensionalVariable<double>('impactcooling','ceel',parameters.ceel);
  _modelFields.AddNonDimensionalVariable<double>('impactcooling','cenl',parameters.cenl);  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactcooling','cebm', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'cebm'));  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactcooling','gdp90', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'gdp90'));  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactcooling','pop90', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'pop90'));  
  
  // Parameters coming from different components, directly coupled
  _modelFields.ConnectParameterToVariable('impactcooling', 'population', 'socioeconomic', 'population');  
  _modelFields.ConnectParameterToVariable('impactcooling', 'income', 'socioeconomic', 'income');  
  _modelFields.ConnectParameterToVariable('impactcooling', 'temp', 'climate', 'temp');  
  
  // Parameters coming from different components, explicitly coupled
  _modelFields.ConnectParameterToVariable('impactcooling', 'cumaeei', 'fundold2impactheatingcooling', 'cumaeei');  
  
  // Create proxy
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactCooling.IImpactCoolingState));
  ImpactCoolingState:=ss.ConnectToState<Fund.Components.ImpactCooling.IImpactCoolingState>(_modelFields, 'impactcooling');
end;

method FundWorkflow.RunImpactCoolingComponent(clock: Clock); 
begin
  var c:=new Fund.Components.ImpactCooling.ImpactCoolingComponent;
            
  // Run ImpactBioDiversity component
  c.Run(clock, ImpactCoolingState);
          
  for each r: Fund.CommonDimensions.Region in Enum.GetValues(typeof(Fund.CommonDimensions.Region)) do
  begin
    _legacyData.economy[ord(r)].cooling:=ImpactCoolingState.cooling[clock.Current,r];
  end; 
end;

method FundWorkflow.InitImpactAgricultureState(parameters: FundParameter); 
begin
  // Constant parameters
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactagriculture','gdp90', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'gdp90'));  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactagriculture','pop90', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'pop90'));  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactagriculture','agrish0', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'agrish0'));  
  _modelFields.AddNonDimensionalVariable<double>('impactagriculture','agel',parameters.agel);
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactagriculture','agrbm', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'agrbm'));  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactagriculture','agtime', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'agtime'));  
  _modelFields.AddNonDimensionalVariable<double>('impactagriculture','agnl',parameters.agnl);
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactagriculture','aglbm', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'aglbm'));  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactagriculture','agopt', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'agopt'));  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactagriculture','agcbm', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'agcbm'));  
  _modelFields.AddNonDimensionalVariable<double>('impactagriculture','co2pre',parameters.co2pre);
  
  // Parameters coming from different components, directly coupled
  _modelFields.ConnectParameterToVariable('impactagriculture', 'income', 'socioeconomic', 'income');  
  _modelFields.ConnectParameterToVariable('impactagriculture', 'population', 'socioeconomic', 'population');    
  _modelFields.ConnectParameterToVariable('impactagriculture', 'temp', 'climate', 'temp');  
  _modelFields.ConnectParameterToVariable('impactagriculture', 'acco2', 'climate', 'acco2');  
    
  // Create proxy
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactAgriculture.IImpactAgricultureState));
  ImpactAgricultureState:=ss.ConnectToState<Fund.Components.ImpactAgriculture.IImpactAgricultureState>(_modelFields, 'impactagriculture');
  
  // Initial values, should eventually also come from parameter file
  var t0:=Timestep.FromSimulationYear(0);
  for each r: Fund.CommonDimensions.Region in Enum.GetValues(typeof(Fund.CommonDimensions.Region)) do
  begin
    ImpactAgricultureState.agrate[t0,r]:=ImpactAgricultureState.agrbm[r]*Math.Pow(0.005/LegacyConstants.dbst,ImpactAgricultureState.agnl)*ImpactAgricultureState.agtime[r];  
  end;
end;

method FundWorkflow.RunImpactAgricultureComponent(clock: Clock); 
begin
  var c:=new Fund.Components.ImpactAgriculture.ImpactAgricultureComponent;
            
  // Run ImpactBioDiversity component
  c.Run(clock, ImpactAgricultureState);
          
  for each r: Fund.CommonDimensions.Region in Enum.GetValues(typeof(Fund.CommonDimensions.Region)) do
  begin
    _legacyData.economy[ord(r)].agcost :=ImpactAgricultureState.agcost[clock.Current,r];
  end; 
end;

method FundWorkflow.InitImpactAggregationState(parameters: FundParameter); 
begin  
  _modelFields.AddNonDimensionalVariable<boolean>('impactaggregation', 'switchoffwater', false);
  _modelFields.AddNonDimensionalVariable<boolean>('impactaggregation', 'switchoffforests', false);
  _modelFields.AddNonDimensionalVariable<boolean>('impactaggregation', 'switchoffheating', false);
  _modelFields.AddNonDimensionalVariable<boolean>('impactaggregation', 'switchoffcooling', false);
  _modelFields.AddNonDimensionalVariable<boolean>('impactaggregation', 'switchoffagcost', false);
  _modelFields.AddNonDimensionalVariable<boolean>('impactaggregation', 'switchoffdrycost', false);
  _modelFields.AddNonDimensionalVariable<boolean>('impactaggregation', 'switchoffprotcost', false);
  _modelFields.AddNonDimensionalVariable<boolean>('impactaggregation', 'switchoffentercost', false);
  _modelFields.AddNonDimensionalVariable<boolean>('impactaggregation', 'switchoffhurrdam', false);
  _modelFields.AddNonDimensionalVariable<boolean>('impactaggregation', 'switchoffspecies', false);
  _modelFields.AddNonDimensionalVariable<boolean>('impactaggregation', 'switchoffdeadcost', false);
  _modelFields.AddNonDimensionalVariable<boolean>('impactaggregation', 'switchoffmorbcost', false);
  _modelFields.AddNonDimensionalVariable<boolean>('impactaggregation', 'switchoffwetcost', false);
  _modelFields.AddNonDimensionalVariable<boolean>('impactaggregation', 'switchoffleavecost', false);
    
  _modelFields.ConnectParameterToVariable('impactaggregation', 'income', 'socioeconomic', 'income');   
  
  _modelFields.ConnectParameterToVariable('impactaggregation', 'heating', 'impactheating', 'heating');
  _modelFields.ConnectParameterToVariable('impactaggregation', 'cooling', 'impactcooling', 'cooling');
  _modelFields.ConnectParameterToVariable('impactaggregation', 'agcost', 'impactagriculture', 'agcost');
  _modelFields.ConnectParameterToVariable('impactaggregation', 'species', 'impactbiodiversity', 'species');  
  _modelFields.ConnectParameterToVariable('impactaggregation', 'water', 'impactwaterresources', 'water');

  _modelFields.ConnectParameterToVariable('impactaggregation', 'forests', 'fundold2impactaggregation', 'forests');
  _modelFields.ConnectParameterToVariable('impactaggregation', 'drycost', 'fundold2impactaggregation', 'drycost');
  _modelFields.ConnectParameterToVariable('impactaggregation', 'protcost', 'fundold2impactaggregation', 'protcost');
  _modelFields.ConnectParameterToVariable('impactaggregation', 'entercost', 'fundold2impactaggregation', 'entercost');
  _modelFields.ConnectParameterToVariable('impactaggregation', 'hurrdam', 'fundold2impactaggregation', 'hurrdam');
  _modelFields.ConnectParameterToVariable('impactaggregation', 'deadcost', 'fundold2impactaggregation', 'deadcost');
  _modelFields.ConnectParameterToVariable('impactaggregation', 'morbcost', 'fundold2impactaggregation', 'morbcost');
  _modelFields.ConnectParameterToVariable('impactaggregation', 'wetcost', 'fundold2impactaggregation', 'wetcost');
  _modelFields.ConnectParameterToVariable('impactaggregation', 'leavecost', 'fundold2impactaggregation', 'leavecost');
    
  // Create proxy
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactAggregation.IImpactAggregationState));
  ImpactAggregationState:=ss.ConnectToState<Fund.Components.ImpactAggregation.IImpactAggregationState>(_modelFields, 'impactaggregation');
  
  // Initial values, should eventually also come from parameter file
  for each r: Fund.CommonDimensions.Region in Enum.GetValues(typeof(Fund.CommonDimensions.Region)) do
  begin  
    ImpactAggregationState.eloss[Timestep.FromSimulationYear(0),r]:=0.0;
    ImpactAggregationState.sloss[Timestep.FromSimulationYear(0),r]:=0.0;
  end;
end;

method FundWorkflow.RunImpactAggregationComponent(clock: Clock); 
begin
  var c:=new Fund.Components.ImpactAggregation.ImpactAggregationComponent;
            
  c.Run(clock, ImpactAggregationState);    
end;

method FundWorkflow.InitFundOld2ImpactAggregationState(parameters: FundParameter); 
begin
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.IFundOld2ImpactAggregation));
  FundOld2ImpactAggregationState:=ss.ConnectToState<Fund.IFundOld2ImpactAggregation>(_modelFields, 'fundold2impactaggregation');
end;

method FundWorkflow.RunFundOld2ImpactAggregationComponent(clock: Clock); 
begin
  var c:=new Fund.FundOld2ImpactAggregation;
            
  c.Run(clock, FundOld2ImpactAggregationState, _legacyData);          
end;

method FundWorkflow.InitGlobalHotellingTaxationState(parameters: FundParameter); 
begin
  _modelFields.AddNonDimensionalVariable<double>('globalhotellingtaxation', 'prtp', 0.03);
  _modelFields.AddNonDimensionalVariable<double>('globalhotellingtaxation', 'elasticityofmarginalutility', 1.0);
  _modelFields.AddNonDimensionalVariable<double>('globalhotellingtaxation','basetax',0.0);
  _modelFields.AddNonDimensionalVariable<Timestep>('globalhotellingtaxation','baseyear',Timestep.FromSimulationYear(50));
  
  _modelFields.ConnectParameterToVariable('globalhotellingtaxation', 'consumption', 'socioeconomic', 'globalconsumption');   
  _modelFields.ConnectParameterToVariable('globalhotellingtaxation', 'population', 'socioeconomic', 'globalpopulation');   
  
  // Create proxy
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.HotellingTaxation.IGlobalHotellingTaxationState));
  GlobalHotellingTaxationState:=ss.ConnectToState<Fund.Components.HotellingTaxation.IGlobalHotellingTaxationState>(_modelFields, 'globalhotellingtaxation');
  
end;

method FundWorkflow.RunGlobalHotellingTaxationComponent(clock: Clock); 
begin
  var c:=new Fund.Components.HotellingTaxation.GlobalHotellingTaxationComponent;
            
  c.Run(clock, GlobalHotellingTaxationState);    
  
end;

method FundWorkflow.ReCreateStateVariables; 
begin
  // Create proxy
  var s1 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.SocioEconomic.ISocioEconomicState));
  SocioEconomicState:=s1.ConnectToState<Fund.Components.SocioEconomic.ISocioEconomicState>(_modelFields, 'socioeconomic');

  var s2 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Welfare.IGlobalWelfareState));
  WelfareState:=s2.ConnectToState<Fund.Components.Welfare.IGlobalWelfareState>(_modelFields, 'globalwelfare');

  var s3 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Welfare.IRegionalWelfareState));
  RegionalWelfareState:=s3.ConnectToState<Fund.Components.Welfare.IRegionalWelfareState>(_modelFields, 'regionalwelfare');
    
  var s4 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactAggregation.IImpactAggregationState));
  ImpactAggregationState:=s4.ConnectToState<Fund.Components.ImpactAggregation.IImpactAggregationState>(_modelFields, 'impactaggregation');

  var s5 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Climate.IClimateState));
  ClimateState:=s5.ConnectToState<Fund.Components.Climate.IClimateState>(_modelFields, 'climate');
    
  var s6 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Ocean.IOceanState));
  OceanState:=s6.ConnectToState<Fund.Components.Ocean.IOceanState>(_modelFields, 'ocean');
  
  var s7 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.BioDiversity.IBioDiversityState));
  BioDiversityState:=s7.ConnectToState<Fund.Components.BioDiversity.IBioDiversityState>(_modelFields, 'biodiversity');
  
  var s8 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactBioDiversity.IImpactBioDiversityState));
  ImpactBioDiversityState:=s8.ConnectToState<Fund.Components.ImpactBioDiversity.IImpactBioDiversityState>(_modelFields, 'impactbiodiversity');
  
  var s9 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.ISocioEconomic2Climate));
  SocioEconomic2ClimateState:=s9.ConnectToState<Fund.ISocioEconomic2Climate>(_modelFields, 'socioeconomic2climate');
  
  var s10 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.ISocioEconomic2ImpactBioDiversityState));
  SocioEconomic2ImpactBioDiversityState:=s10.ConnectToState<Fund.ISocioEconomic2ImpactBioDiversityState>(_modelFields, 'socioeconomic2impactbiodiversity');
  
  var s11 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.IFundOld2SocioEconomic));
  FundOld2SocioEconomicState:=s11.ConnectToState<Fund.IFundOld2SocioEconomic>(_modelFields, 'fundold2socioeconomic');
  
  var s12 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.IFundOld2Climate));
  FundOld2ClimateState:=s12.ConnectToState<Fund.IFundOld2Climate>(_modelFields, 'fundold2climate');
  
  var s13 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.ISocioEconomic2Welfare));
  FundOld2GlobalWelfareState:=s13.ConnectToState<Fund.ISocioEconomic2Welfare>(_modelFields, 'fundold2globalwelfare');
  
  var s14 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.ISocioEconomic2RegionalWelfare));
  SocioEconomic2RegionalWelfareState:=s14.ConnectToState<Fund.ISocioEconomic2RegionalWelfare>(_modelFields, 'socioeconomic2regionalwelfare');
  
  var s15 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ScenarioUncertainty.IScenarioUncertaintyState));
  ScenarioUncertaintyState:=s15.ConnectToState<Fund.Components.ScenarioUncertainty.IScenarioUncertaintyState>(_modelFields, 'scenariouncertainty');
  
  var s16 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactHeating.IImpactHeatingState));
  ImpactHeatingState:=s16.ConnectToState<Fund.Components.ImpactHeating.IImpactHeatingState>(_modelFields, 'impactheating');
  
  var s17 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactCooling.IImpactCoolingState));
  ImpactCoolingState:=s17.ConnectToState<Fund.Components.ImpactCooling.IImpactCoolingState>(_modelFields, 'impactcooling');
  
  var s18 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.IFundOld2ImpactHeatingCoolingState));
  FundOld2ImpactHeatingCoolingState:=s18.ConnectToState<Fund.IFundOld2ImpactHeatingCoolingState>(_modelFields, 'fundold2impactheatingcooling');
  
  var s19 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactAgriculture.IImpactAgricultureState));
  ImpactAgricultureState:=s19.ConnectToState<Fund.Components.ImpactAgriculture.IImpactAgricultureState>(_modelFields, 'impactagriculture');
  
  var s20 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.IFundOld2ImpactAggregation));
  FundOld2ImpactAggregationState:=s20.ConnectToState<Fund.IFundOld2ImpactAggregation>(_modelFields, 'fundold2impactaggregation');
  
  var s21 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.HotellingTaxation.IGlobalHotellingTaxationState));
  GlobalHotellingTaxationState:=s21.ConnectToState<Fund.Components.HotellingTaxation.IGlobalHotellingTaxationState>(_modelFields, 'globalhotellingtaxation');

  var s22 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactWaterResources.IImpactWaterResourcesState));
  ImpactWaterResourcesState:=s22.ConnectToState<Fund.Components.ImpactWaterResources.IImpactWaterResourcesState>(_modelFields, 'impactwaterresources');

  var s23 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeOf(Fund.Components.ImpactDiarrhoea.IImpactDiarrhoeaState));
  ImpactDiarrhoeaState := s23.ConnectToState<Fund.Components.ImpactDiarrhoea.IImpactDiarrhoeaState>(_modelFields, 'impactdiarrhoea');

  var s24 := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeOf(Fund.Components.ClimateRegional.IClimateRegionalState));
  ClimateRegionalState := s24.ConnectToState<Fund.Components.ClimateRegional.IClimateRegionalState>(_modelFields, 'climateregional');
end;

method FundWorkflow.InitImpactWaterResourcesState(parameters: FundParameter);
begin
  // Constant parameters
  _modelFields.AddNonDimensionalVariable<double>('impactwaterresources','watechrate',parameters.watechrate);
  _modelFields.AddNonDimensionalVariable<double>('impactwaterresources','wrel',parameters.wrel);  
  _modelFields.AddNonDimensionalVariable<double>('impactwaterresources','wrnl',parameters.wrnl);  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactwaterresources','wrbm', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'wrbm'));  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactwaterresources','gdp90', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'gdp90'));  
  
  // Parameters coming from different components, directly coupled
  _modelFields.ConnectParameterToVariable('impactwaterresources', 'income', 'socioeconomic', 'income');  
  _modelFields.ConnectParameterToVariable('impactwaterresources', 'temp', 'climate', 'temp');  
    
  // Create proxy
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactWaterResources.IImpactWaterResourcesState));
  ImpactWaterResourcesState:=ss.ConnectToState<Fund.Components.ImpactWaterResources.IImpactWaterResourcesState>(_modelFields, 'impactwaterresources');
end;

method FundWorkflow.RunImpactWaterResourcesComponent(clock: Clock);
begin
  var c:=new Fund.Components.ImpactWaterResources.ImpactWaterResourcesComponent;
            
  c.Run(clock, ImpactWaterResourcesState);
          
  for each r: Fund.CommonDimensions.Region in Enum.GetValues(typeof(Fund.CommonDimensions.Region)) do
  begin
    _legacyData.economy[ord(r)].water:=ImpactWaterResourcesState.water[clock.Current,r];
  end; 
end;

method FundWorkflow.InitImpactDiarrhoeaState(parameters: FundParameter);
begin
  // Constant parameters
  _modelFields.AddNonDimensionalVariable<double>('impactdiarrhoea','diamortel',parameters.diamortel);
  _modelFields.AddNonDimensionalVariable<double>('impactdiarrhoea','diamortnl',parameters.diamortnl);
  _modelFields.AddNonDimensionalVariable<double>('impactdiarrhoea','diayldel',parameters.diayldel);
  _modelFields.AddNonDimensionalVariable<double>('impactdiarrhoea','diayldnl',parameters.diayldnl);

  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactdiarrhoea','diamort', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'diamort'));  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactdiarrhoea','diayld', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'diayld'));  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactdiarrhoea','gdp90', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'gdp90'));  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactdiarrhoea','pop90', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'pop90'));  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactdiarrhoea','temp90', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'temp90'));  
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('impactdiarrhoea','bregtmp', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'bregtmp'));  
  
  // Parameters coming from different components, directly coupled
  _modelFields.ConnectParameterToVariable('impactdiarrhoea', 'income', 'socioeconomic', 'income');  
  _modelFields.ConnectParameterToVariable('impactdiarrhoea', 'population', 'socioeconomic', 'population');  
  _modelFields.ConnectParameterToVariable('impactdiarrhoea', 'temp', 'climate', 'temp');  
  _modelFields.ConnectParameterToVariable('impactdiarrhoea', 'regtmp', 'climateregional', 'regtmp');
    
  // Create proxy
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactDiarrhoea.IImpactDiarrhoeaState));
  ImpactDiarrhoeaState:=ss.ConnectToState<Fund.Components.ImpactDiarrhoea.IImpactDiarrhoeaState>(_modelFields, 'impactdiarrhoea');
end;

method FundWorkflow.RunImpactDiarrhoeaComponent(clock: Clock);
begin
  var c:=new Fund.Components.ImpactDiarrhoea.ImpactDiarrhoeaComponent;
            
  c.Run(clock, ImpactDiarrhoeaState);
          
  for each r: Fund.CommonDimensions.Region in Enum.GetValues(typeof(Fund.CommonDimensions.Region)) do
  begin
    _legacyData.economy[ord(r)].diadead:=ImpactDiarrhoeaState.diadead[clock.Current,r];
    _legacyData.Economy[ord(r)].diasick:=ImpactDiarrhoeaState.diasick[clock.Current,r];
  end; 
end;

method FundWorkflow.InitClimateRegionalState(parameters: FundParameter);
begin
  // Constant parameters
  _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region,double>('climateregional','bregtmp', new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, 'bregtmp'));  
  
  // Parameters coming from different components, directly coupled
  _modelFields.ConnectParameterToVariable('climateregional', 'temp', 'climate', 'temp');  
    
  // Create proxy
  var ss := Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ClimateRegional.IClimateRegionalState));
  ClimateRegionalState:=ss.ConnectToState<Fund.Components.ClimateRegional.IClimateRegionalState>(_modelFields, 'climateregional');
end;

method FundWorkflow.RunClimateRegionalComponent(clock: Clock);
begin
  var c:=new Fund.Components.ClimateRegional.ClimateRegionalComponent;
            
  c.Run(clock, ClimateRegionalState);
          
  // Calculate regional temperatures  
  for each r: Fund.CommonDimensions.Region in Enum.GetValues(typeof(Fund.CommonDimensions.Region)) do
  begin
    _legacyData.economy[ord(r)].regtmp:=ClimateRegionalState.regtmp[clock.Current,r];
    _legacyData.economy[ord(r)].regdt:=ClimateRegionalState.regtmp[clock.Current,r]-ClimateRegionalState.regtmp[clock.Current-1,r];
    
    if _legacyData.economy[ord(r)].regdt < 0 then
    begin
      raise new Exception('regdt got negative, this is not handled properly');
      // In particular, this is not handled in ClimateRegional at all
      _legacyData.economy[ord(r)].regdt:= -_legacyData.economy[ord(r)].regdt;
    end;
  end;

end;

end.