// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

uses Esmf, System.Collections.Generic,
     System.Reflection.Emit;

type
  
  YearRegionParameterConnector<D2> = public class (Esmf.IParameter2Dimensional<Esmf.Timestep,D2,double>)
  private
    _values: array [0..LegacyConstants.NYear+1,0..15] of double;
    _enumConversionDelegate: Enum2IntConversionDelegate<D2>;
  protected
    method GetItem(t: Esmf.Timestep; d: D2): Double;
  public
    constructor(param: Parameters; name: string);
    
    property Item[t: Esmf.Timestep; d: D2]: double read GetItem; default;
    method GetEnumerator: System.Collections.Generic.IEnumerable<Esmf.Parameter2DimensionalMember<Esmf.Timestep,D2,System.Double>>;
  end;
   
implementation

method YearRegionParameterConnector<d2>.GetItem(t: Esmf.Timestep; d: D2): Double; 
begin
  Result:=_values[t.Value,_enumConversionDelegate(d)-1];
end;

constructor YearRegionParameterConnector<d2>(param: Parameters; name: string); 
begin 
  for i:integer:=0 to LegacyConstants.NYear do
  begin
    for each r: integer in array of Integer(Enum.GetValues(typeof(D2))) do
    begin
      _values[i, r-1]:=param.GetItemByYearAndRegion(i,Enum.GetName(typeof(D2),r),name);
    end;

    _enumConversionDelegate:=Enum2IntConversionSingleton<D2>.Instance.Delegate;
    
  end;
end;

method YearRegionParameterConnector<D2>.GetEnumerator: System.Collections.Generic.IEnumerable<Esmf.Parameter2DimensionalMember<Esmf.Timestep,D2,System.Double>>;
begin
  raise new NotImplementedException;
end;

end.