// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

uses
  System.Configuration,
  System.IO;

type
  TotalDamage = public class
  private
    m_YearRegionCsv: TextWriter;
    m_YearRegionWeightingschemeCsv: TextWriter;
    m_YearRegionSectorWeightingSchemeCsv: TextWriter;
    m_AggregateDamageCsv: TextWriter;
    m_GlobalInputCsv: TextWriter;
    m_RegionInputCsv: TextWriter;
    m_YearInputCsv: TextWriter;
    m_RegionYearInputCsv: TextWriter;    
    
    _run: Run;
    _outputPath: String;
    _parameters: FundParameter;
    
    procedure DoOneRun(RunId: Integer; i_weightingCombinations: array of WeightingCombination);    
    procedure MargRun(Output: ModelOutput);
    
    
    method WriteAggregateDamage(RunId: Integer; WeightingschemeId: Integer; Damage: Double; WeightingCombinations: array of WeightingCombination);
    method WriteDamage(RunId: Integer; i_Damage: Damage; WeightingschemeId: Integer; Weight: Double; WeightingCombinations: array of WeightingCombination);
    method WriteYearRegionWeightingschemeData(RunId: Integer; RegionId: Integer; WeightingSchemeId: Integer; i_weightingCombinations: array of WeightingCombination; Year: Integer);
    method WriteYearRegionData(RunId: Integer; Year: Integer; Region: Integer; Income: Double; Population: Double);
  
  protected
  public
    constructor(runData: Run; outputPath: String; parameters: FundParameter);

    procedure Run;

    property AggregateDamageCsv: Textwriter read m_AggregateDamageCsv write m_AggregateDamageCsv;
    property YearRegionCsv: TextWriter read m_YearRegionCsv write m_YearRegionCsv;
    property YearRegionWeightingSchemeCsv: TextWriter read m_YearRegionWeightingschemeCsv write m_YearRegionWeightingschemeCsv;
    property YearRegionSectorWeightingSchemeCsv: TextWriter read m_YearRegionSectorWeightingSchemeCsv write m_YearRegionSectorWeightingSchemeCsv;
    property GlobalInputCsv: TextWriter read m_GlobalInputCsv write m_GlobalInputCsv;
    property RegionInputCsv: TextWriter read m_RegionInputCsv write m_RegionInputCsv;
    property YearInputCsv: TextWriter read m_YearInputCsv write m_YearInputCsv;
    property RegionYearInputCsv: TextWriter read m_RegionYearInputCsv write m_RegionYearInputCsv;    
  end;
  
implementation

method TotalDamage.WriteAggregateDamage(RunId: Integer; WeightingschemeId: Integer; Damage: Double; WeightingCombinations: array of WeightingCombination); 
begin
  if m_AggregateDamageCsv<>nil then
  begin
    m_AggregateDamageCsv.Write(_run.Scenario.Id);
    m_AggregateDamageCsv.Write(';');

    if _run.OutputVerbal then
    begin
      case Convert.ToInt32(_parameters.marginalGas) of
        0: m_AggregateDamageCsv.Write('"C"');
        else
          raise new NotImplementedException('Only carbon supported for output at this point');
      end;
    end
    else
      m_AggregateDamageCsv.Write(Convert.ToInt32(_parameters.marginalGas));

    m_AggregateDamageCsv.Write(';');
    m_AggregateDamageCsv.Write(Convert.ToInt32(_parameters.emissionPeriod));
    m_AggregateDamageCsv.Write(';');

    if _run.OutputVerbal then
    begin
      if RunId=0 then
        m_AggregateDamageCsv.Write('"Best guess"')
      else
        m_AggregateDamageCsv.Write(RunId);
    end
    else
      m_AggregateDamageCsv.Write(RunId);

    m_AggregateDamageCsv.Write(';');

    if _run.OutputVerbal then
    begin
      m_AggregateDamageCsv.Write('"' + WeightingCombinations[WeightingschemeId].Name + '"');
    end
    else
      m_AggregateDamageCsv.Write(WeightingschemeId);

    m_AggregateDamageCsv.Write(';');
    m_AggregateDamageCsv.Write('{0:f15}', Object(Damage));
    m_AggregateDamageCsv.WriteLine;

  end;
end;

method TotalDamage.WriteDamage(RunId: Integer; i_Damage: Damage; WeightingschemeId: Integer; Weight: Double; WeightingCombinations: array of WeightingCombination); 
var
  i_e: double;
begin
  if m_YearRegionSectorWeightingSchemeCsv<>nil then
  begin
    m_YearRegionSectorWeightingSchemeCsv.Write(_run.Scenario.Id);
    m_YearRegionSectorWeightingSchemeCsv.Write(';');
       
    if _run.OutputVerbal then
    begin
      if RunId=0 then
        m_YearRegionSectorWeightingSchemeCsv.Write('"Best guess"')
      else
        m_YearRegionSectorWeightingSchemeCsv.Write(RunId);
    end
    else
      m_YearRegionSectorWeightingSchemeCsv.Write(RunId);
      
    // Marginal Gas and emissions year are empty for total runs
    m_YearRegionSectorWeightingSchemeCsv.Write(';;');
    
    m_YearRegionSectorWeightingSchemeCsv.Write(';');
    m_YearRegionSectorWeightingSchemeCsv.Write((i_Damage.Year + 1950));
    m_YearRegionSectorWeightingSchemeCsv.Write(';');
    
    if _run.OutputVerbal then
    begin
      m_YearRegionSectorWeightingSchemeCsv.Write(LegacyRegionNames.Name[i_Damage.Region]);
    end
    else
      m_YearRegionSectorWeightingSchemeCsv.Write(i_Damage.Region);
      
    m_YearRegionSectorWeightingSchemeCsv.Write(';');
    
    if _run.OutputVerbal then
    begin
      m_YearRegionSectorWeightingSchemeCsv.Write(Enum.GetName(typeof(Sector), i_Damage.Sector));
    end
    else
      m_YearRegionSectorWeightingSchemeCsv.Write(Ord(i_Damage.Sector));
      
    m_YearRegionSectorWeightingSchemeCsv.Write(';');
    
    if _run.OutputVerbal then
    begin
      m_YearRegionSectorWeightingSchemeCsv.Write('"' + WeightingCombinations[WeightingschemeId].Name + '"');
    end
    else
      m_YearRegionSectorWeightingSchemeCsv.Write(WeightingschemeId);
    
    m_YearRegionSectorWeightingSchemeCsv.Write(';');
    i_e := i_Damage.Damage * Weight;
    m_YearRegionSectorWeightingSchemeCsv.Write('{0:f15}', Object(i_e));
    m_YearRegionSectorWeightingSchemeCsv.WriteLine;
  end;
end;

method TotalDamage.WriteYearRegionWeightingschemeData(RunId: Integer; RegionId: Integer; WeightingSchemeId: Integer; i_weightingCombinations: array of WeightingCombination; Year: Integer); 
var
  i_e: double;
begin
  if m_YearRegionWeightingschemeCsv<>nil then
  begin
    m_YearRegionWeightingschemeCsv.Write(_run.Scenario.Id);
    m_YearRegionWeightingschemeCsv.Write(';');
    m_YearRegionWeightingschemeCsv.Write(Convert.ToInt32(_parameters.marginalGas));
    m_YearRegionWeightingschemeCsv.Write(';');
    m_YearRegionWeightingschemeCsv.Write(Convert.ToInt32(_parameters.emissionPeriod));
    m_YearRegionWeightingschemeCsv.Write(';');
    m_YearRegionWeightingschemeCsv.Write(RunId);
    m_YearRegionWeightingschemeCsv.Write(';');
    m_YearRegionWeightingschemeCsv.Write((Year + 1950));
    m_YearRegionWeightingschemeCsv.Write(';');
    m_YearRegionWeightingschemeCsv.Write(RegionId);
    m_YearRegionWeightingschemeCsv.Write(';');
    m_YearRegionWeightingschemeCsv.Write(WeightingSchemeId);
    m_YearRegionWeightingschemeCsv.Write(';');
    i_e := i_weightingCombinations[WeightingSchemeId].Weight[Year, RegionId];
    m_YearRegionWeightingschemeCsv.Write('{0:f15}', Object(i_e));
    m_YearRegionWeightingschemeCsv.WriteLine;
  end;
end;

method TotalDamage.WriteYearRegionData(RunId: Integer; Year: Integer; Region: Integer; Income: Double; Population: Double); 
begin
  if m_YearRegionCsv<>nil then
  begin
    m_YearRegionCsv.Write(_run.Scenario.Id);
    m_YearRegionCsv.Write(';');
    m_YearRegionCsv.Write(Convert.ToInt32(_parameters.marginalGas));
    m_YearRegionCsv.Write(';');
    m_YearRegionCsv.Write(Convert.ToInt32(_parameters.emissionPeriod));
    m_YearRegionCsv.Write(';');
    m_YearRegionCsv.Write(RunId);
    m_YearRegionCsv.Write(';');
    m_YearRegionCsv.Write((Year + 1950));
    m_YearRegionCsv.Write(';');
    m_YearRegionCsv.Write(Region);
    m_YearRegionCsv.Write(';');
    m_YearRegionCsv.Write('{0:f15}', Object(Income));
    m_YearRegionCsv.Write(';');
    m_YearRegionCsv.Write('{0:f15}', Object(Population));
    m_YearRegionCsv.WriteLine;
  end;
end;

constructor TotalDamage(runData: Run; outputPath: String; parameters: FundParameter); 
begin
  _parameters:=parameters;
  _run:=runData;
  _outputPath:=outputPath;
end;

procedure TotalDamage.Run; 
var
  i: Integer;
  i_weightingCombinations: array of WeightingCombination;
  lDimWeightingschemeCsv: TextWriter;
begin
  _parameters.SetToBestGuessValue;
  
  Fund28LegacyWeightingCombinations.GetWeightingCombinationsFromName(_run.WeightingCombination, out i_weightingCombinations, _parameters);

  lDimWeightingschemeCsv:=new StreamWriter(Path.Combine(_outputPath, 'Output - Dim Weightingscheme.csv'));
  for i:=0 to High(i_weightingCombinations) do
  begin
    lDimWeightingschemeCsv.WriteLine(i.ToString + ';' + i_weightingCombinations[i].Name + '');
  end;
  lDimWeightingschemeCsv.Close;

  for i:=0 TO _run.MonteCarloRuns DO
  begin
    if i=0 then
    begin
      Console.WriteLine('Best guess run ');

      _parameters.SetToBestGuessValue;
      _parameters.WriteValuesToCsv(_run.Scenario.Id, i,
           m_GlobalInputCsv, m_RegionInputCsv, m_YearInputCsv, m_RegionYearInputCsv);      
    end
    else
    begin
      Console.WriteLine('Run ' + i.ToString);

      _parameters.SetToRandomValue;
      _parameters.WriteValuesToCsv(_run.Scenario.Id, i,
           m_GlobalInputCsv, m_RegionInputCsv, m_YearInputCsv, m_RegionYearInputCsv);      
    end;

    DoOneRun(i, i_weightingCombinations);
  end;
end;

procedure TotalDamage.DoOneRun(RunId: Integer; i_weightingCombinations: array of WeightingCombination); 
var
  i: Integer;
  i_Damage: Damage;
  i_aggregatedDamage: double;
  l: Integer;
  k: Integer;
  m: Integer;
  i_output1: ModelOutput;
begin
  // Create Output object for run 1, set addmp to 0 so that
  // the extra greenhouse gases are not emitted and then run
  // the model
  i_output1 := new ModelOutput;
  MargRun(i_output1);

  for i := 0 to High(i_weightingCombinations) do
  begin
    i_weightingCombinations[i].CalculateWeights(i_output1);
    i_aggregatedDamage := i_weightingCombinations[i].AddDamagesUp(i_output1.Damages, _run.YearsToAggregate, Convert.ToInt32(_parameters.emissionPeriod));

    WriteAggregateDamage(RunId, i, i_aggregatedDamage, i_weightingCombinations);

  end;
  
  for l := 0 to i_output1.Damages.Count - 1 do
  begin
    i_Damage := i_output1.Damages.Item[l];
    if (i_Damage.Year >= _parameters.emissionPeriod-1950) AND (i_Damage.Year < _parameters.emissionPeriod-1950 + _run.YearsToAggregate) then
    begin
      for k := 0 to High(i_weightingCombinations) do
        WriteDamage(RunId, i_Damage, k, i_weightingCombinations[k].Weight[i_Damage.Year, i_Damage.Region], i_weightingCombinations );
    end;
  end;
  for l := Convert.ToInt32(_parameters.emissionPeriod-1950) to Convert.ToInt32(_parameters.emissionPeriod-1950)-1950 + _run.YearsToAggregate do
  begin
    for k := 1 to LegacyConstants.NoReg do
    begin
      WriteYearRegionData(RunId, l, k, i_output1.Incomes.Item[l, k], i_output1.Populations.Item[l, k]);
      for m := 0 to High(i_weightingCombinations) do
        WriteYearRegionWeightingschemeData(RunId, k, m, i_weightingCombinations, l);
    end;
  end;
end;

procedure TotalDamage.MargRun(Output: ModelOutput); 
var
  fundWorkflow: FundWorkflow;
begin
  fundWorkflow:=new FundWorkflow(_parameters);
  
  fundWorkflow.Run(Output);         
end;

end.