// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

type
  AnthoffEquityWeighting = public class (WeightingScheme )
  private
    m_baseYear: Integer;
    m_baseRegion: integer;
    m_inequalityAversion: double;
  protected
    function GetName: string; override;
  public
    constructor(BaseYear: integer; BaseRegion: string; InequalityAversion: Double);

    procedure CalculateWeights(ModelOutput: ModelOutput); override;
  end;
  
implementation

{ AnthoffEquityWeighting }

procedure AnthoffEquityWeighting.CalculateWeights(
  ModelOutput: ModelOutput);
var
  i_totalIncome, i_totalPopulation: double;
  i_baseYpC: double;
  i_YpC: double;
  i, l: Integer;
begin
  inherited;
  if m_baseRegion=-1 then
  begin
    i_totalIncome:=0;
    i_totalPopulation:=0;
    for i:=1 to LegacyConstants.NoReg do
    begin
      i_totalIncome:=i_totalIncome + ModelOutput.Incomes.Item[m_baseYear, i];
      i_totalPopulation:=i_totalPopulation + ModelOutput.Populations.Item[m_baseYear, i];
    end;
    i_baseYpC:=i_totalIncome/i_totalPopulation;
  end
  else
    i_baseYpC:=Modeloutput.Incomes.Item[m_baseYear, m_baseRegion] / ModelOutput.Populations.Item[m_baseYear, m_baseRegion];

  for i:=1 to LegacyConstants.NoReg do
  begin
    // TODO -oDavid Anthoff: Change start of "real" period to constant
    for l:=50 to LegacyConstants.NYear do
    begin
      i_YpC:=Modeloutput.Incomes.Item[l, i] / ModelOutput.Populations.Item[l, i];
      AddWeight(l, i, LegacyFund28Math.pow(i_baseYpC/i_YpC, m_inequalityAversion));
    end;
  end;
end;

constructor AnthoffEquityWeighting(BaseYear: integer;
  BaseRegion: string; InequalityAversion: Double);
var
  i: integer;
  i_regionIndex: integer;
begin
  inherited constructor;

  i_regionIndex:=-1;
  for i:=1 to LegacyConstants.NoReg do
  begin
    if LegacyRegionNames.Name[i]=BaseRegion then
    begin
      i_regionIndex:=i;
    end;
  end;

  m_baseYear:=BaseYear;
  m_baseRegion:=i_regionIndex;
  m_inequalityAversion:=InequalityAversion;
end;

function AnthoffEquityWeighting.GetName: string;
begin
  if m_baseRegion=-1 then
    // TODO -oDavid Anthoff: Exchange 1950 with constant
    Result:='utility (in ' + Convert.ToString(m_baseYear+1950) + ' units, mu=' + Convert.ToString(m_inequalityAversion) + ')'
  else
    Result:='utility (in ' + Convert.ToString(m_baseYear+1950) + ', ' + LegacyRegionNames.Name[m_baseRegion] + ' units, mu=' + Convert.ToString(m_inequalityAversion) + ')';
end;

end.