// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

type
  GlobalGrowthDiscounterateWeighting = public class (WeightingScheme)
  private
    m_baseYear: Integer;
    m_inequalityAversion: double;
  protected
  public
    constructor(BaseYear: integer; InequalityAversion: Double);
    method CalculateWeights(ModelOutput: Fund.ModelOutput); override;
    method GetName: System.String; override;
  end;
  
implementation

method GlobalGrowthDiscounterateWeighting.CalculateWeights(ModelOutput: Fund.ModelOutput);
var
  i_totalIncome, i_totalPopulation: double;
  i_baseYpC: double;
  i_YpC: double;
  i, l: Integer;
begin
  i_totalIncome:=0;
  i_totalPopulation:=0;
  for i:=1 to LegacyConstants.NoReg do
  begin
    i_totalIncome:=i_totalIncome + ModelOutput.Incomes.Item[m_baseYear, i];
    i_totalPopulation:=i_totalPopulation + ModelOutput.Populations.Item[m_baseYear, i];
  end;
  i_baseYpC:=i_totalIncome/i_totalPopulation;

  for l:=m_baseYear to LegacyConstants.NYear do
  begin
    var i_income:=0.0;
    var i_population:=0.0;
    for i:=1 to LegacyConstants.NoReg do
    begin
      i_income:=i_income+Modeloutput.Incomes.Item[l, i];
      i_population:=i_population+ModelOutput.Populations.Item[l, i];
    end;
    
    i_YpC:=i_income / i_population;
    
    for i:=1 to LegacyConstants.NoReg do
    begin
      
      AddWeight(l, i, Math.Pow(i_baseYpC/i_YpC, m_inequalityAversion));
    end;
    
  end;
end;

method GlobalGrowthDiscounterateWeighting.GetName: System.String;
begin
  Result:='global utility (in ' + Convert.ToString(m_baseYear+1950) + ' units, mu=' + Convert.ToString(m_inequalityAversion) + ')';  
end;

constructor GlobalGrowthDiscounterateWeighting(BaseYear: integer; InequalityAversion: Double); 
begin
  m_baseYear:=BaseYear;
  m_inequalityAversion:=InequalityAversion;
end;

end.