// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

type
  GollierUncertainGrowthDiscounting = public class(WeightingScheme)
  private
    m_upperBound, m_lowerBound, m_prtp: double;
  protected
    function GetName: string; override;
  public
    constructor(Prtp, UpperBound, LowerBound: double);

    procedure CalculateWeights(ModelOutput: ModelOutput); override;
  end;
  
implementation

{ GollierUncertainGrowthDiscounting }

procedure GollierUncertainGrowthDiscounting.CalculateWeights(
  ModelOutput: ModelOutput);
var
  i_discountFactorUpper: double;
  i_discountFactorLower: double;
  i_newPerCapitaIncome, i_oldPerCapitaIncome: double;
  i_perCapitaGrowthRate: double;
  i, l: integer;
begin
  for i:=1 to LegacyConstants.NoReg do
  begin
    i_discountFactorUpper:=1;
    i_discountFactorLower:=1;
    for l:=50 to LegacyConstants.NYear do
    begin
      AddWeight(l, i, (i_discountFactorUpper+i_discountFactorLower)/2);

      i_newPerCapitaIncome:=ModelOutput.Incomes.Item[l, i]/ModelOutput.Populations.Item[l, i];
      i_oldPerCapitaIncome:=ModelOutput.Incomes.Item[l-1, i]/ModelOutput.Populations.Item[l-1, i];
      i_perCapitaGrowthRate:=(i_newPerCapitaIncome-i_oldPerCapitaIncome)/i_oldPerCapitaIncome;


      i_discountFactorUpper:=i_discountFactorUpper/(1 + m_prtp + i_perCapitaGrowthRate + m_upperBound );
      i_discountFactorLower:=i_discountFactorLower/(1 + m_prtp + i_perCapitaGrowthRate - m_lowerBound );
    end;
  end;

end;

constructor GollierUncertainGrowthDiscounting(Prtp, UpperBound,
  LowerBound: double);
begin
  inherited constructor;
  m_prtp:=Prtp;
  m_upperBound:=UpperBound;
  m_lowerBound:=LowerBound;
end;

function GollierUncertainGrowthDiscounting.GetName: string;
begin
  Result:='Gollier (prtp=' + Convert.ToString(m_prtp*100) + '%, boundary=' + Convert.ToString(m_upperBound*100) + '%)';
end;

end.