// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

type
  GollierUncertainGrowthDiscountingNormalDistribution = public class(WeightingScheme)
  private
    m_Sigma, m_prtp: double;
  protected
    function GetName: string; override;
  public
    constructor(Prtp, Sigma: double);

    procedure CalculateWeights(ModelOutput: ModelOutput); override;
  end;
  
implementation

{ GollierUncertainGrowthDiscountingNormalDistribution }

procedure GollierUncertainGrowthDiscountingNormalDistribution.CalculateWeights(
  ModelOutput: ModelOutput);
var
  i_discountFactor1: double;
  i_discountFactor2: double;
  i_discountFactor3: double;
  i_discountFactor4: double;
  i_discountFactor5: double;
  i_newPerCapitaIncome, i_oldPerCapitaIncome: double;
  i_perCapitaGrowthRate: double;
  i, l: integer;
begin
  for i:=1 to LegacyConstants.NoReg do
  begin
    i_discountFactor1:=1;
    i_discountFactor2:=1;
    i_discountFactor3:=1;
    i_discountFactor4:=1;
    i_discountFactor5:=1;
    for l:=50 to LegacyConstants.NYear do
    begin
      AddWeight(l, i, i_discountFactor1*0.0227+i_discountFactor2*0.136+i_discountFactor3*0.6826+i_discountFactor4*0.136+i_discountFactor5*0.0227);

      i_newPerCapitaIncome:=ModelOutput.Incomes.Item[l, i]/ModelOutput.Populations.Item[l, i];
      i_oldPerCapitaIncome:=ModelOutput.Incomes.Item[l-1, i]/ModelOutput.Populations.Item[l-1, i];
      i_perCapitaGrowthRate:=(i_newPerCapitaIncome-i_oldPerCapitaIncome)/i_oldPerCapitaIncome;

      i_discountFactor1:=i_discountFactor1/(1 + m_prtp + i_perCapitaGrowthRate - 2.5*m_Sigma);
      i_discountFactor2:=i_discountFactor2/(1 + m_prtp + i_perCapitaGrowthRate - 1.5*m_Sigma);
      i_discountFactor3:=i_discountFactor3/(1 + m_prtp + i_perCapitaGrowthRate);
      i_discountFactor4:=i_discountFactor4/(1 + m_prtp + i_perCapitaGrowthRate + 1.5*m_Sigma);
      i_discountFactor5:=i_discountFactor5/(1 + m_prtp + i_perCapitaGrowthRate + 2.5*m_Sigma);
    end;
  end;
end;

constructor GollierUncertainGrowthDiscountingNormalDistribution(
  Prtp, Sigma: double);
begin
  inherited constructor;
  m_prtp:=Prtp;
  m_Sigma:=Sigma;
end;

function GollierUncertainGrowthDiscountingNormalDistribution.GetName: string;
begin
  Result:='Gollier normal (prtp=' + Convert.ToString(m_prtp*100) + '%, sigma=' + Convert.ToString(m_Sigma*100) + '%)';
end;

end.