// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

type
  // Declining discount rate according to the UK green book
  GreenbookDiscounting = public class( WeightingScheme )
  private
    m_baseYear: Integer;
  protected
    function GetName: string; override;
  public
    constructor(BaseYear: Integer);

    procedure CalculateWeights(ModelOutput: ModelOutput); override;
  end;
  
implementation

{ GreenbookDiscounting }

procedure GreenbookDiscounting.CalculateWeights(ModelOutput: ModelOutput);
var
  i, l: Integer;
  i_discountFactor: double;
  i_discountRate: double;
begin
  inherited;
  i_discountFactor:=1;

  for i:=0 to LegacyConstants.NYear-m_baseYear do
  begin
    for l:=1 to LegacyConstants.NoReg do
      AddWeight(i +  m_baseYear, l, i_discountFactor);

    if i<30 then
      i_discountRate:=0.035
    else if i<75 then
      i_discountRate:=0.03
    else if i<125 then
      i_discountRate:=0.025
    else if i<200 then
      i_discountRate:=0.02
    else if i<300 then
      i_discountRate:=0.015
    else
      i_discountRate:=0.01;
    i_discountFactor:= i_discountFactor/(1.00 + i_discountRate);
  end;
end;

constructor GreenbookDiscounting(BaseYear: Integer);
begin
  inherited constructor;

  m_baseYear:=BaseYear;
end;

function GreenbookDiscounting.GetName: string;
begin
  Result:='greenbook discounting (base year: ' + m_baseYear.ToString + ')';
end;

end.