// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

type
  // Declining discounting according to Weitzman
  WeitzmanDiscounting = public class (WeightingScheme)
  private
    mMju, mSigma: double;
  protected
    function GetName: string; override;
  public
    constructor(Mju, Sigma: double);

    procedure CalculateWeights(ModelOutput: ModelOutput); override;
  end;
  
implementation

{ WeitzmanDiscounting }

procedure WeitzmanDiscounting.CalculateWeights(ModelOutput: ModelOutput);
var
  i_discountFactor: double;
  i_discountRate: double;
  i, l: Integer;
begin
  inherited;
  i_discountFactor:=1;
  for i:=50 to LegacyConstants.NYear do
  begin
    for l:=1 to LegacyConstants.NoReg do
    begin
      AddWeight(i, l, i_discountFactor);
    end;

    i_discountRate:=mMju/(1+(i-50)*(LegacyFund28Math.pow(mSigma,2)/mMju));

    i_discountFactor:=i_discountFactor/(1+i_discountRate);
  end;
end;

constructor WeitzmanDiscounting(Mju, Sigma: double);
begin
  inherited constructor;
  mMju:=Mju;
  mSigma:=Sigma;
end;

function WeitzmanDiscounting.GetName: string;
begin
  Result:='Weitzman discounting';
end;

end.